;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                        ;;
;;  VESA12.INC                                            ;;
;;                                                        ;;
;;  Vesa 1.2 functions for MenuetOS ver 0.75              ;;
;;                                                        ;;
;;  Copyright 2002 Ville Turjanmaa                        ;;
;;                                                        ;;
;;  quickcode@mail.ru -                                   ;;
;;                                                        ;;
;;  See file COPYING for details                          ;;
;;                                                        ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; A complete video driver should include the following types of function
;
; Putpixel
; Getpixel
;
; Drawimage
; Drawbar
;
; Drawbackground
;
;
; Modified for Cirrus CL-GD54X VGA and ATI Mach64
;
; May 10, 2004 D.E.Berglind

set_bank:

  cli
  cmp  al,[0xfff2] ;32K block change?
  je   retsb
  mov  [0xfff2],al
  push ax
  push dx
  mov  edx,eax	;Mach64 has 32 bit registers
  shl  eax,16
  or   eax,edx
  mov  dx,0x56ec	;Mem_VGA_Wp_Sel
  out  dx,eax
  mov  dx,0x5aec	:Mem_VGA_Rp_Sel
  out  dx,eax
  shr  eax,1
  mov  ah,al
  cmp  al,[0xfff3] ;64K block change?
  je   retsb-4
  mov  [0xfff3],al
  mov  dx,0x03c4  ;Sequencer Index Register
  mov  al,0x10    ;SR6 "Unlock All Extensions Register"
  out  dx,al
  inc  dl
  mov  al,0x12    ;Unlock Extention Registers
  out  dx,al
  mov  dx,0x03ce  ;VGA Graphics Controller Index Register
  mov  al,0x09    ;GR9 "Offset Register 0"
  out  dx,al      ;register has 4K granularity
  inc  dl
  mov  al,ah      ;Bank offset value
  shl  ax,4       ;shift to select 64K segment
  out  dx,al
  mov  dx,0x03c4  ;Sequencer Index Register
  mov  al,0x10     ;SR6 "Unlock All Extensions Register"
  out  dx,al
  inc  dl
  mov  al,0x11    ;Lock Extention Registers
  out  dx,al
  pop  dx
  pop  ax

  retsb:

  ret


vesa12_drawbackground:

 push  eax
 push  ebx
 push  ecx
 push  edx

 xor   edx,edx
 mov   eax,dword[0x400000-8]  ;image x size
 mov   ebx,dword[0x400000-4]  ;image y size
 mul   ebx
 mov   ebx,3
 mul   ebx
 mov   [imax],eax
 mov   eax,[0xd000+32+0]  ;screen x start position
 mov   ebx,[0xd000+32+4]  ;screen y start position
 mov   edi,0 ;no force

      v12dp3:

 push  eax      ;current x position
 push  ebx      ;current y position
 xor   edx,edx
 mov   ecx,[0x400000-12]
 cmp   ecx,dword 1            ;tiled or stretched?
 je    v12tile
 mov   ecx,dword [0x400000-8] ;stretch image
 mul   ecx                    ;screen x position * image width
 mov   ecx,[0xfe00]           ;divided by screen width
 div   ecx                    ; = image x position
 push  eax
 xor   edx,edx
 mov   ecx,dword [0x400000-4] ;image hight
 mov   eax,ebx
 mul   ecx                    ;screen y position * image hight
 mov   ecx,[0xfe04]           ;divided by screen hight
 div   ecx                    ; = image y position
 jmp   v12bg1

      v12tile:                ;tile image

 mov   ecx,dword [0x400000-8] ;screen x position / image width
 div   ecx                    ;eax = frame no.
 push  edx                    ;edx = displacement into N+1 frame
 xor   edx,edx
 mov   ecx,dword [0x400000-4] ;screen y position / image hight
 mov   eax,ebx                ;eax = frame no.
 div   ecx                    ;edx = displacement into N+1 frame
 mov   eax,edx

 v12bg1:

 mov   ebx,dword [0x400000-8] ;adjust for three bytes per pixel
 add   ebx,dword [0x400000-8]
 add   ebx,dword [0x400000-8] ;3 * image width
 mul   ebx                    ; * y position
 mov   esi,eax
 pop   eax                    ; plus 3 * x position
 add   esi,eax
 add   esi,eax
 add   esi,eax
 add   esi,0x300000           ;plus background image base add.
 pop   ebx
 pop   eax

 v12di4:

 mov   ecx,[esi]
 pusha
 mov   esi,eax
 mov   edi,ebx
 mov   eax,[0xfe00]
 add   eax,1
 mul   ebx
 add   eax,esi
 add   eax,0x400000
 cmp   [eax],byte 1
 jnz   v12nbgp
 mov   eax,[0xfe08]
 mov   ebx,edi
 mul   ebx
 add   eax,esi
 add   eax,esi
 add   eax,esi
 cmp   [0xFBF1],byte 24
 jz    v12bgl3
 add   eax,esi

 v12bgl3:

 push  ebx
 push  eax
 shr   eax,15
 call  set_bank
 pop   eax
 and   eax,0xffff
 add   eax,0xa0000
 pop   ebx
 mov   [eax],cx
 add   eax,2
 shr   ecx,16
 mov   [eax],cl
 sti

 v12nbgp:

 popa
 add   esi,3
 inc   eax
 cmp   eax,[0xd000+32+8]  ;screen x size
 jg    v12nodp31
 jmp   v12dp3

 v12nodp31:

 mov   eax,[0xd000+32+0]  ;screen x start position
 inc   ebx
 cmp   ebx,[0xd000+32+12] ;screen y size
 jg    v12dp4
 jmp   v12dp3

 v12dp4:

 pop   edx
 pop   ecx
 pop   ebx
 pop   eax
 ret


 vesa12_drawbar:


 mov  [novesachecksum],dword 0
 sub  edx,ebx
 sub  ecx,eax
 push ebp
 push esi
 push edi
 push eax
 push ebx
 push ecx
 push edx
 mov  ecx,[0x3010]
 add  eax,[ecx-0x1000]
 add  ebx,[ecx-0x1000+4]
 push eax
 mov  eax,ebx ; y
 mov  ebx,[0xfe08]
 mul  ebx
 pop  ecx
 add  eax,ecx ; x
 add  eax,ecx
 add  eax,ecx
 cmp  [0xfbf1],byte 24     ; 24 or 32 bpp ? - x start
 jz   dbpi2412
 add  eax,ecx


 dbpi2412:

 add eax,[0xfe80]
 mov edi,eax
 mov ebx,[esp+0]

    ; check limits ?

 mov eax,[0x3010]
 mov ecx,[eax+0xd000-0x3000+0]
 cmp ecx,0
 jnz dbcblimitlset12
 mov ecx,[eax+0xd000-0x3000+4]
 cmp ecx,0
 jnz dbcblimitlset12
 mov ecx,[eax+0xd000-0x3000+8]
 cmp ecx,[0xfe00]
 jnz dbcblimitlset12
 mov ecx,[eax+0xd000-0x3000+12]
 cmp ecx,[0xfe04]
 jnz dbcblimitlset12
 push dword 0
 jmp dbcblimitlno12

 dbcblimitlset12:

 push dword 1
 dbcblimitlno12:

 cmp [0xfbf1],byte 24     ; 24 or 32 bpp ?
 jz dbpi24bit12
 jmp dbpi32bit12


; DRAWBAR 24 BBP


 dbpi24bit12:

 mov  ecx,[esp+8] ;bar width
 cld

 dbnewpi12:

 push ebx
 push edi
 push ecx
 xor edx,edx
 mov eax,edi
 sub eax,[0xfe80]     ;LBF base
 mov ebx,[0xfe08]     ;bytes per line
 div ebx              ;find line number
 mov ebx,[0xfe00]     ;screen x size
 inc ebx
 mul ebx
 add eax,0x400000     ;screen data base
 mov ebx,[0x3010]     ;process base
 add eax,[ebx-0x1000] ;x start value
 movzx ebx,byte[ebx+0xe]
 cld

      dbnp2412:

 mov dl,[eax]
 push eax
 push ecx
 cmp dl,bl
 jnz dbimp24no12
 cmp [esp+5*4],dword 0
 jz dbimp24yes12
 call dbcplimit
 jnz dbimp24no12

 dbimp24yes12:

 push edi
 mov eax,edi
 sub eax,[0xfe80]
 shr eax,15
 call set_bank
 and edi,0xffff
 add edi,0xa0000
 mov eax,[esp+8+3*4+16+4+4]
 stosw
 shr eax,16
 stosb
 sti
 pop edi
 add edi,3
 pop ecx
 pop eax
 inc eax
 loop dbnp2412
 jmp dbnp24d12

      dbimp24no12:

 add edi,3
 pop ecx
 pop eax
 cld
 inc eax
 loop dbnp2412

 dbnp24d12:

 mov  eax,[esp+3*4+16+4]
 test eax,0x80000000
 jz   nodbgl2412
 cmp  al,0
 jz   nodbgl2412
 dec  eax
 mov  [esp+3*4+16+4],eax

 nodbgl2412:

 pop  ecx
 pop  edi
 pop  ebx
 add  edi,[0xfe08]
 dec  ebx
 jz dbnonewpi12
 jmp  dbnewpi12

  dbnonewpi12:

 pop  ebp
 add  esp,7*4
 ret


; DRAWBAR 32 BBP


 dbpi32bit12:

 mov  ecx,[esp+8] ;bar width
 cld

 dbnewpi3212:

 push ebx
 push edi
 push ecx
 xor  edx,edx
 mov eax,edi
 sub eax,[0xfe80]     ;LBF base
 mov ebx,[0xfe08]     ;bytes/line
 div ebx              ;find line number
 mov ebx,[0xfe00]     ;screen x size
 inc ebx
 mul ebx
 add eax,0x400000     ;screen data base
 mov ebx,[0x3010]     ;process base
 add eax,[ebx-0x1000] ;x start value
 movzx ebx,byte[ebx+0xe]
 cld

 dbnp3212:

 mov  dl,[eax]
 push eax
 push ecx
 cmp  dl,bl
 jnz  dbimp32no12
 cmp  [esp+5*4],dword 0
 jz   dbimp32yes12
 call dbcplimit
 jnz  dbimp32no12

 dbimp32yes12:

 push edi
 mov  eax,edi
 sub  eax,[0xfe80]
 shr  eax,15
 call set_bank
 and  edi,0xffff
 add  edi,0xa0000
 mov  eax,[esp+8+3*4+16+4+4]
 stosw
 shr  eax,16
 stosb
 sti
 pop  edi
 add  edi,4
 inc  ebp
 pop  ecx
 pop  eax
 inc  eax
 loop dbnp3212
 jmp  dbnp32d12

 dbimp32no12:

 pop  ecx
 pop  eax
 inc  eax
 add  edi,4
 inc  ebp
 loop dbnp3212

 dbnp32d12:

 mov  eax,[esp+12+16+4]
 test eax,0x80000000
 jz   nodbgl3212
 cmp  al,0
 jz   nodbgl3212
 dec  eax
 mov  [esp+12+16+4],eax

 nodbgl3212:

 pop  ecx
 pop  edi
 pop  ebx
 add  edi,[0xfe08]
 dec  ebx
 jz  nodbnewpi3212
 jmp  dbnewpi3212

 nodbnewpi3212:

 add  esp,7*4
 ret


Vesa12_putpixel24:

 mov  edi,eax ; x
 mov  eax,ebx ; y
 lea  edi,[edi+edi*2]
 mov  ebx,[0xfe08]
 mul  ebx
 add  edi,eax
 mov  eax,edi
 shr  eax,15
 call set_bank
 and  edi,0xffff
 add  edi,0xa0000
 mov  eax,[esp+28]
 stosw
 shr  eax,16
 mov  [edi],al
 sti
 ret



Vesa12_putpixel32:

 mov  edi,eax ; x
 mov  eax,ebx ; y
 shl  edi,2
 mov  ebx,[0xfe08]
 mul  ebx
 add  edi,eax
 mov  eax,edi
 shr  eax,15
 call set_bank
 and  edi,0xffff
 add  edi,0xa0000
 mov  ecx,[esp+28]
 mov  [edi],ecx
 sti
 ret


Vesa12_getpixel24:

 mov  edi,eax ; x
 mov  eax,ebx ; y
 lea  edi,[edi+edi*2]
 mov  ebx,[0xfe08]
 mul  ebx
 add  edi,eax
 mov  eax,edi
 shr  eax,15
 call set_bank
 and  edi,0xffff
 add  edi,0xa0000
 mov  ecx,[edi]
 and  ecx,255*256*256+255*256+255
 sti
 ret


Vesa12_getpixel32:

 mov  edi,eax ; x
 mov  eax,ebx ; y
 shl  edi,2
 mov  ebx,[0xfe08]
 xor  edx,edx
 mul  ebx
 add  edi,eax
 mov  eax,edi
 shr  eax,15
 call set_bank
 and  edi,0xffff
 add  edi,0xa0000
 mov  ecx,[edi]
 and  ecx,255*256*256+255*256+255
 sti

 ret



vesa12_putimage:

;    mov  ebx,image
;    mov  ecx,320*65536+240
;    mov  edx,20*65536+20


 mov   [novesachecksum],dword 0
 push  esi
 push  edi
 push  eax
 push  ebx
 push  ecx
 push  edx
 movzx eax,word [esp+2]
 movzx ebx,word [esp+0]
 mov   ecx,[0x3010]
 add   eax,[ecx-0x1000]
 add   ebx,[ecx-0x1000+4]
 push  eax
 mov   eax,ebx   ; y
 mov   ebx,[0xfe08]
 mul   ebx
 pop   ecx
 add   eax,ecx   ; x
 add   eax,ecx
 add   eax,ecx
 cmp  [0xfbf1],byte 24     ; 24 or 32 bpp ? - x start
 jz pi2412
 add  eax,ecx

 pi2412:

 add  eax,[0xfe80]
 mov  edi,eax

    ; x size


 pi24312:

 mov   esi,[esp+8]
 movzx ebx,word [esp+4]

    ; check limits while draw ?

 push  eax
 push  ecx
 mov  eax,[0x3010]
 mov  ecx,[eax+0xd000-0x3000+0]
 cmp  ecx,0
 jnz  dbcblimitlset212
 mov  ecx,[eax+0xd000-0x3000+4]
 cmp  ecx,0
 jnz  dbcblimitlset212
 mov  ecx,[eax+0xd000-0x3000+8]
 cmp  ecx,[0xfe00]
 jnz  dbcblimitlset212
 mov  ecx,[eax+0xd000-0x3000+12]
 cmp  ecx,[0xfe04]
 jnz  dbcblimitlset212
 pop  ecx
 pop  eax
 push dword 0
 jmp  dbcblimitlno212

 dbcblimitlset212:

 pop  ecx
 pop  eax
 push dword 1

 dbcblimitlno212:

 cmp  [0xfbf1],byte 24     ; 24 or 32 bpp ?
 jz pi24bit12
 jmp  pi32bit12

 pi24bit12:

 cld
 movzx  ecx,word[esp+10]

 newpi12:

 push edi
 push esi
 push ecx
 push ebx

 xor edx,edx
 mov eax,edi
 sub eax,[0xfe80]     ;LBF base
 mov ebx,[0xfe08]     ;bytes/line
 div ebx              ;find line number
 mov ebx,[0xfe00]     ;screen x size
 inc ebx
 mul ebx
 add eax,0x400000     ;screen data base
 mov ebx,[0x3010]     ;process base
 add eax,[ebx-0x1000] ;x start value
 mov bl,[ebx+0xe]
 mov bh,[esp+4*4]

 np2412:

 cmp  bl,[eax]
 jnz  imp24no12
 mov  edx,[esi]
 cmp  bh,0
 jz   imp24yes12
 call dbcplimit
 jnz  imp24no12

 imp24yes12:

 push eax
 push edi
 mov  eax,edi
 sub  eax,[0xfe80]
 shr  eax,15
 call set_bank
 and  edi,0xffff
 add  edi,0xa0000
 mov  [edi],dx
 shr  edx,16
 mov  [edi+2],dl
 sti
 pop  edi
 pop  eax

 imp24no12:

 inc  eax
 add  esi,3
 add  edi,3
 dec  ecx
 jnz  np2412

 np24d12:

 pop  ebx
 pop  ecx
 pop  esi
 pop  edi
 add  edi,[0xfe08]
 xor  eax,eax
 mov  ax,[esp+4+2+4]
 lea  eax,[eax+eax*2]
 add  esi,eax
 dec  ebx
 jz  nonewpi12
 jmp  newpi12

 nonewpi12:

 add esp,7*4
 mov  eax,0
 ret


 pi32bit12:

 cld
 movzx  ecx,word[esp+10]

 newpi3212:

 push edi
 push esi
 push ecx
 push ebx

 xor edx,edx
 mov eax,edi
 sub eax,[0xfe80]     ;LBF base
 mov ebx,[0xfe08]     ;bytes/line
 div ebx              ;find line number
 mov ebx,[0xfe00]     ;screen x size
 inc ebx
 mul ebx
 add eax,0x400000     ;screen data base
 mov ebx,[0x3010]     ;process base
 add eax,[ebx-0x1000] ;x start value
 mov bl,[ebx+0xe]
 mov bh,[esp+4*4]

 np3212:

 cmp  bl,[eax]
 jnz  imp32no12
 mov  edx,[esi]
 cmp  bh,0
 jz   imp32yes12
 call dbcplimit
 jnz  imp32no12

 imp32yes12:

 push eax
 push edi
 mov  eax,edi
 sub  eax,[0xfe80]
 shr  eax,15
 call set_bank
 and  edi,0xffff
 add  edi,0xa0000
 mov  [edi],edx
 sti
 pop  edi
 pop  eax

 imp32no12:

 inc  eax
 add  esi,3
 add  edi,4
 dec  ecx
 jnz  np3212

 np32d12:

 pop  ebx
 pop  ecx
 pop  esi
 pop  edi

 add   edi,[0xfe08]
 movzx eax,word [esp+4+2+4]
 lea   eax,[eax+eax*2]
 add   esi,eax
 dec   ebx
 jz nonewpi3212
 jmp   newpi3212

 nonewpi3212:

 add   esp,7*4
 mov   eax,0
 ret


vesa12_read_screen_pixel:

 and   eax,0x3FFFFF
 cmp   [0xfbf1],byte 24  ; 24 or 32 bpp ?
 jz    v12rsp24
 mov   edi,eax
 shl   edi,2
 mov   eax,edi
 shr   eax,15
 call  set_bank
 and   edi,0xffff
 add   edi,0xa0000
 mov   eax,[edi]
 and   eax,0x00ffffff
 ret

 v12rsp24:

 imul  eax,3
 mov   edi,eax
 shr   eax,15
 call  set_bank
 and   edi,0xffff
 add   edi,0xa0000
 mov   eax,[edi]
 and   eax,0x00ffffff
 ret

          ;this is the end